package sun.misc;

import java.util.Iterator;
import java.util.Arrays;
import java.util.ArrayList;
import java.io.IOException;
import java.io.EOFException;
import java.io.InputStream;

public class IOUtils
{
    private static final int DEFAULT_BUFFER_SIZE = 8192;
    private static final int MAX_BUFFER_SIZE = 2147483639;
    
    public IOUtils() {
    }
    
    public static byte[] readExactlyNBytes(InputStream inputStream, int n) throws IOException {
        byte[] nBytesOrEOF = readNBytesOrEOF(inputStream, n);
        if (nBytesOrEOF.length < n) {
            throw new EOFException();
        }
        return nBytesOrEOF;
    }
    
    public static byte[] readNBytesOrEOF(InputStream inputStream, int i) throws IOException {
        if (i < 0) {
            throw new IOException("length cannot be negative: " + i);
        }
        Object o = null;
        byte[] original = null;
        int n = 0;
        int read;
        do {
            byte[] b;
            int off;
            for (b = new byte[Math.min(i, 8192)], off = 0; (read = inputStream.read(b, off, Math.min(b.length - off, i))) > 0; off += read, i -= read) {}
            if (off > 0) {
                if (2147483639 - n < off) {
                    throw new OutOfMemoryError("Required array size too large");
                }
                n += off;
                if (original == null) {
                    original = b;
                }
                else {
                    if (o == null) {
                        o = new ArrayList();
                        o.add(original);
                    }
                    o.add(b);
                }
            }
            else {
                continue;
            }
        } while (read >= 0 && i > 0);
        if (o == null) {
            if (original == null) {
                return new byte[0];
            }
            return (original.length == n) ? original : Arrays.copyOf(original, n);
        }
        else {
            byte[] array = new byte[n];
            int n2 = 0;
            for (byte[] array2 : o) {
                int min = Math.min(array2.length, n);
                System.arraycopy(array2, 0, array, n2, min);
                n2 += min;
                n -= min;
            }
            return array;
        }
    }
    
    public static byte[] readFully(InputStream inputStream, int n, boolean b) throws IOException {
        byte[] original = new byte[0];
        if (n == -1) {
            n = Integer.MAX_VALUE;
        }
        int i = 0;
        while (i < n) {
            int min;
            if (i >= original.length) {
                min = Math.min(n - i, original.length + 1024);
                if (original.length < i + min) {
                    original = Arrays.copyOf(original, i + min);
                }
            }
            else {
                min = original.length - i;
            }
            int read = inputStream.read(original, i, min);
            if (read < 0) {
                if (b && n != Integer.MAX_VALUE) {
                    throw new EOFException("Detect premature EOF");
                }
                if (original.length != i) {
                    original = Arrays.copyOf(original, i);
                    break;
                }
                break;
            }
            else {
                i += read;
            }
        }
        return original;
    }
    
    public static byte[] readNBytes(InputStream inputStream, int i) throws IOException {
        if (i < 0) {
            throw new IOException("length cannot be negative: " + i);
        }
        return readFully(inputStream, i, true);
    }
}
